
SVH.p <- function(Vs, SDs = T, ns, Reps = 10000, seed = 1)
{
   # program to calculate Standardized Variance Heterogeneity statistic plus a p value
   # Vs = vector of variances (or SDs, see below) for the groups
   # SDs = whether you've provided SDs (the default setting) or variances (in which case set SDs = F)
   # ns = vector of sample sizes for the groups
   # Reps = number of samples of artificial comparison data generated to calculate p value (default = 10,000)
   # seed = random number seed (default = 1)
   
   # count number of groups and, if applicable, convert SDs to variances
   k <- length(Vs)
   if (SDs) Vs <- Vs ^ 2

   # calculate SVH for empirical data
   kV <- k * Vs / sum(Vs)
   SVH.e <- sd(kV) * sqrt((k - 1) / k) / sqrt(k - 1)
 
   # generate artificial comparison data sets and calculate SVH for each
   set.seed(seed)
   Vs.c <- rep(0, k)
   SVH.c <- rep(0, Reps)
   for (i in 1:Reps)
   {
      for (j in 1:k)
         Vs.c[j] <- var(rnorm(ns[j]))
      kV <- k * Vs.c / sum(Vs.c)
      SVH.c[i] <- sd(kV) * sqrt((k - 1) / k) / sqrt(k - 1)
   }

   # calculate p value for the empirical SVH value
   p <- 1 - (sum(SVH.e > SVH.c) / Reps)

   # print results
   cat("SVH =", SVH.e, "\n")
   cat("p =", round(p,ceiling(log10(Reps))), "\n")

   # plot distribution of SVH values for comparison data, with VR and SVH in title 
   # and empirical SVH and p value indicated in the graph
   dSVH <- density(SVH.c)
   plot(dSVH, xlim = c(0,1), xlab = "Standardized Variance Heterogeneity (SVH)", 
      main = "Distribution of SVH Values for Artificial Comparison Data\nSVH for Empirical Data Plotted as Vertical Line")
   x <- SVH.e
   y <- max(dSVH$y)
   lines(x = c(x,x), y = c(0,y*.90), lty = 3)
}


